#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "commonTypes.h"
#include "ringbuffer.h"



typedef struct {
    bool stopSignal;
    ringbuffer_t *ringbuffer;
} mariadbsinkThreadContext_t;

void mariadbsinkStop(void *ptr) {
    commonThreadHandle_t *handle = (commonThreadHandle_t*)ptr;
    mariadbsinkThreadContext_t *context = (mariadbsinkThreadContext_t*)handle->sinkContext;

    context->stopSignal = true;
    fprintf(stderr, "mariadb sink thread, stop flagged\n");
}

mariadbsinkThreadContext_t *_mariadbsinkInit(commonThreadHandle_t *handle) {
    mariadbsinkThreadContext_t *context = (mariadbsinkThreadContext_t*) malloc(sizeof(mariadbsinkThreadContext_t));
    handle->sinkContext = (void*)context;

    context->stopSignal = false;
    context->ringbuffer = handle->ringbuffer;

    return context;
}

int _mariadbsinkExec(mariadbsinkThreadContext_t *context) {
    while (! context->stopSignal) {
        auditItem_t *auditItem = (auditItem_t*)ringbufferGet(context->ringbuffer);

        if (auditItem != NULL) {
            printf("AuditItem: Time:    %ld\n", auditItem->ts);
            printf("AuditItem: Topic:   %s\n", auditItem->topic);
            printf("AuditItem: Payload: %s\n", auditItem->payload);

            free(auditItem->topic);
            auditItem->topic = NULL;
            free(auditItem->payload);
            auditItem->payload = NULL;
            free(auditItem);
            auditItem = NULL;
        }
    }
    return 0;
}

int _mariadbsinkDeInit(commonThreadHandle_t *handle) {
    free((mariadbsinkThreadContext_t*)handle->sinkContext);
    handle->sinkContext = NULL;
    return 0;
}




void *mariadbsink(void *ptr) {
    fprintf(stderr, "mariadb sink entered\n");

    mariadbsinkThreadContext_t *context = _mariadbsinkInit((commonThreadHandle_t*)ptr);
    
    _mariadbsinkExec(context);

    _mariadbsinkDeInit((commonThreadHandle_t*)ptr);

    fprintf(stderr, "mariadb sink thread stopped\n");
    return (void*)NULL;
}

